@@ -1,25 +1,28 @@
-/* mpc
- * (c) 2003-2004 by normalperson and Warren Dukes (warren.dukes@gmail.com)
- *              and Daniel Brown (danb@cs.utexas.edu)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include "libmpdclient.h"
 #include "list.h"
-#include "charConv.h"
+#include "charset.h"
 #include "password.h"
 #include "util.h"
 #include "status.h"
@@ -46,7 +49,7 @@ int funcname(mpd_unused int argc, mpd_unused char **argv, \
 
 #define SIMPLE_ONEARG_CMD(funcname, libmpdclient_funcname, ret) \
 int funcname (mpd_unused int argc, char **argv, mpd_Connection *conn) { \
-        libmpdclient_funcname(conn, toUtf8(argv[0])); \
+        libmpdclient_funcname(conn, charset_to_utf8(argv[0])); \
         my_finishCommand(conn); \
         return ret; \
 }
@@ -81,105 +84,22 @@ static mpd_Status * getStatus(mpd_Connection * conn) {
 	return ret;
 }
 
-static int isUrl(char * s) {
-	char * t;
-
-	t = strstr(s,"://");
-	if(!t) return 0;
-
-	while(s!=t) {
-		if(!(*s >= 'A' && *s <= 'z')) return 0;
-		s++;
-	}
-
-	return 1;
-}
-
-int cmd_add (int argc, char ** argv, mpd_Connection * conn ) 
+int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 {
-	List ** lists;
-	mpd_InfoEntity * entity;
-	ListNode * node;
-	struct mpd_song *song;
-	char * sp;
-	char * duplicated;
 	int i;
-	int * arglens;
-	int len;
-	int ret;
 
-	lists = malloc(sizeof(List *)*(argc));
-	arglens = malloc(sizeof(int)*(argc));
-
-	/* convert ' ' to '_' */
-	for(i=0;i<argc;i++) {
-		lists[i] = makeList(free);
-		sp = argv[i];
-		while((sp = strchr(sp,' '))) *sp = '_';
-		arglens[i] = strlen(argv[i]);
-	}
-
-	 /* get list of songs to add */
-	 mpd_sendListallCommand(conn,"");
-	 printErrorAndExit(conn);
-
-	while((entity = mpd_getNextInfoEntity(conn))) {
-		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
-			song = entity->info.song;
-			sp = duplicated = strdup(fromUtf8(song->file));
-			while((sp = strchr(sp,' '))) *sp = '_';
-			len = strlen(duplicated);
-			for(i=0;i<argc;i++) {
-				if (strcmp(argv[i], "/") == 0) {
-					insertInListWithoutKey(lists[i],
-					                       strdup(fromUtf8(song->file)));
-					continue;
-				}
-				if(len<arglens[i]) continue;
-				ret = strncmp(argv[i], duplicated,
-						arglens[i]);
-				if(ret==0 && (argv[i][arglens[i]-1] == '/' ||
-					      duplicated[arglens[i]] == '\0' ||
-					      duplicated[arglens[i]] == '/'))
-				{
-					insertInListWithoutKey(
-							lists[i],
-							strdup(fromUtf8(
-									song->file)));
-				}
-			}
-			free(duplicated);
-		}
-		mpd_freeInfoEntity(entity);
-	}
-	my_finishCommand(conn);
-
-	/* send list of songs */
 	mpd_sendCommandListBegin(conn);
 	printErrorAndExit(conn);
-	for(i=0;i<argc;i++) {
-		if(0 == lists[i]->numberOfNodes && isUrl(argv[i])) {
-			printf("adding: %s\n",argv[i]);
-			mpd_sendAddCommand(conn,toUtf8(argv[i]));
-			printErrorAndExit(conn);
-			continue;
-		}	
 
-		node = lists[i]->firstNode;
-		while(node) {
-			printf("adding: %s\n",(char *)node->data);
-			mpd_sendAddCommand(conn,toUtf8(node->data));
-			printErrorAndExit(conn);
-			node = node->nextNode;
-		}
-		freeList(lists[i]);
+	for(i=0;i<argc;i++) {
+		printf("adding: %s\n", argv[i]);
+		mpd_sendAddCommand(conn, charset_to_utf8(argv[i]));
+		printErrorAndExit(conn);
 	}
+
 	mpd_sendCommandListEnd(conn);
 	my_finishCommand(conn);
 
-	/* clean up */
-	free(lists);
-	free(arglens);
 	return 0;
 }
 
@@ -216,12 +136,60 @@ cmd_crop(mpd_unused int argc, mpd_unused char **argv, mpd_Connection *conn)
 
 	} else {
 
-		mpd_freeStatus(status);	
+		mpd_freeStatus(status);
 		DIE( "You need to be playing to crop the playlist\n" );
 
 	}
 }
 
+int cmd_current(mpd_unused int argc, mpd_unused char ** argv, mpd_Connection *conn)
+{
+	mpd_Status * status;
+	mpd_InfoEntity * entity;
+
+	mpd_sendCommandListOkBegin(conn);
+	printErrorAndExit(conn);
+	mpd_sendStatusCommand(conn);
+	printErrorAndExit(conn);
+	mpd_sendCurrentSongCommand(conn);
+	printErrorAndExit(conn);
+	mpd_sendCommandListEnd(conn);
+	printErrorAndExit(conn);
+
+	status = mpd_getStatus(conn);
+	printErrorAndExit(conn);
+
+	if (status->state == MPD_STATUS_STATE_PLAY ||
+	    status->state == MPD_STATUS_STATE_PAUSE) {
+		mpd_nextListOkCommand(conn);
+		printErrorAndExit(conn);
+
+		while((entity = mpd_getNextInfoEntity(conn))) {
+			struct mpd_song *song = entity->info.song;
+
+			if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
+				mpd_freeInfoEntity(entity);
+				continue;
+			}
+
+			pretty_print_song(song);
+			printf("\n");
+
+			mpd_freeInfoEntity(entity);
+
+			break;
+		}
+
+		printErrorAndExit(conn);
+
+		mpd_finishCommand(conn);
+		printErrorAndExit(conn);
+	}
+	mpd_freeStatus(status);
+
+	return 0;
+}
+
 int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
 {
 	int i,j;
@@ -595,7 +563,8 @@ int cmd_listall ( int argc, char ** argv, mpd_Connection * conn )
 	const char * listall = "";
 	int i=0;
 
-	if(argc>0) listall = toUtf8(argv[i]);
+	if (argc > 0)
+		listall = charset_to_utf8(argv[i]);
 
 	do {
 		mpd_sendListallCommand(conn,listall);
@@ -605,12 +574,12 @@ int cmd_listall ( int argc, char ** argv, mpd_Connection * conn )
 
 		my_finishCommand(conn);
 
-	} while(++i<argc&& (listall = toUtf8(argv[i])));
+	} while (++i < argc && (listall = charset_to_utf8(argv[i])) != NULL);
 
 	return 0;
 }
 
-int cmd_update ( int argc, char ** argv, mpd_Connection * conn) 
+int cmd_update ( int argc, char ** argv, mpd_Connection * conn)
 {
 	const char * update = "";
 	int i = 0;
@@ -618,11 +587,11 @@ int cmd_update ( int argc, char ** argv, mpd_Connection * conn)
 	mpd_sendCommandListBegin(conn);
 	printErrorAndExit(conn);
 
-	if(argc > 0) update = toUtf8(argv[i]);
+	if(argc > 0) update = charset_to_utf8(argv[i]);
 
 	do {
 		mpd_sendUpdateCommand(conn, update);
-	} while(++i<argc && (update = toUtf8(argv[i])));
+	} while (++i < argc && (update = charset_to_utf8(argv[i])) != NULL);
 
 	mpd_sendCommandListEnd(conn);
 	printErrorAndExit(conn);
@@ -635,57 +604,33 @@ int cmd_update ( int argc, char ** argv, mpd_Connection * conn)
 int cmd_ls ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
-	char * ls;
+	const char *ls;
 	int i = 0;
-	char * sp;
-	char * dp;
 
 	if (argc > 0)
-		ls = toUtf8(argv[i]);
+		ls = charset_to_utf8(argv[i]);
 	else
 		ls = strdup("");
 
 	do {
-		mpd_sendListallCommand(conn,"");
-
-		sp = ls;
-		while((sp = strchr(sp,' '))) *sp = '_';
-
-		while((entity = mpd_getNextInfoEntity(conn))) {
-			if(entity->type==MPD_INFO_ENTITY_TYPE_DIRECTORY) {
-				mpd_Directory * dir = entity->info.directory;
-				sp = dp = strdup(dir->path);
-				while((sp = strchr(sp,' '))) *sp = '_';
-				if(strcmp(dp,ls)==0) {
-					free(dp);
-					ls = dir->path;
-					break;
-				}
-				free(dp);
-			}
-			mpd_freeInfoEntity(entity);
-		}
-
-		my_finishCommand(conn);
-
 		mpd_sendLsInfoCommand(conn,ls);
 		printErrorAndExit(conn);
 
 		while((entity = mpd_getNextInfoEntity(conn))) {
 			if(entity->type==MPD_INFO_ENTITY_TYPE_DIRECTORY) {
 				mpd_Directory * dir = entity->info.directory;
-				printf("%s\n",fromUtf8(dir->path));
+				printf("%s\n", charset_from_utf8(dir->path));
 			}
 			else if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
 				struct mpd_song *song = entity->info.song;
-				printf("%s\n",fromUtf8(song->file));
+				printf("%s\n", charset_from_utf8(song->file));
 			}
 			mpd_freeInfoEntity(entity);
 		}
 
 		my_finishCommand(conn);
 
-	} while(++i<argc && (ls = toUtf8(argv[i])));
+	} while (++i < argc && (ls = charset_to_utf8(argv[i])) != NULL);
 
 	return 0;
 }
@@ -696,7 +641,7 @@ int cmd_lsplaylists ( int argc, char ** argv, mpd_Connection * conn )
 	const char * ls = "";
 	int i = 0;
 
-	if(argc>0) ls = toUtf8(argv[i]);
+	if(argc>0) ls = charset_to_utf8(argv[i]);
 
 	do {
 		mpd_sendLsInfoCommand(conn,ls);
@@ -706,14 +651,14 @@ int cmd_lsplaylists ( int argc, char ** argv, mpd_Connection * conn )
 			if(entity->type==
 					MPD_INFO_ENTITY_TYPE_PLAYLISTFILE) {
 				mpd_PlaylistFile * pl = entity->info.playlistFile;
-				printf("%s\n",fromUtf8(pl->path));
+				printf("%s\n", charset_from_utf8(pl->path));
 			}
 			mpd_freeInfoEntity(entity);
 		}
 
 		my_finishCommand(conn);
 
-	} while(++i<argc && (ls = toUtf8(argv[i])));
+	} while (++i < argc && (ls = charset_to_utf8(argv[i])) != NULL);
 	return 0;
 }
 
@@ -735,11 +680,11 @@ int cmd_load ( int argc, char ** argv, mpd_Connection * conn )
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_PLAYLISTFILE) {
 			pl = entity->info.playlistFile;
-			dp = sp = strdup(fromUtf8(pl->path));
+			dp = sp = strdup(charset_from_utf8(pl->path));
 			while((sp = strchr(sp,' '))) *sp = '_';
 			for(i=0;i<argc;i++) {
 				if(strcmp(dp,argv[i])==0)
-					strcpy(argv[i], fromUtf8(pl->path));
+					strcpy(argv[i], charset_from_utf8(pl->path));
 			}
 			free(dp);
 			mpd_freeInfoEntity(entity);
@@ -751,7 +696,7 @@ int cmd_load ( int argc, char ** argv, mpd_Connection * conn )
 	printErrorAndExit(conn);
 	for(i=0;i<argc;i++) {
 		printf("loading: %s\n",argv[i]);
-		mpd_sendLoadCommand(conn,toUtf8(argv[i]));
+		mpd_sendLoadCommand(conn,charset_to_utf8(argv[i]));
 		printErrorAndExit(conn);
 	}
 	mpd_sendCommandListEnd(conn);
@@ -760,7 +705,7 @@ int cmd_load ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-static int do_search ( int argc, char ** argv, mpd_Connection * conn, int exact ) 
+static int do_search ( int argc, char ** argv, mpd_Connection * conn, int exact )
 {
 	Constraint *constraints;
 	int numconstraints;
@@ -777,7 +722,7 @@ static int do_search ( int argc, char ** argv, mpd_Connection * conn, int exact
 
 	for (i = 0; i < numconstraints; i++) {
 		mpd_addConstraintSearch(conn, constraints[i].type,
-		                        toUtf8(constraints[i].query));
+		                        charset_to_utf8(constraints[i].query));
 	}
 
 	free(constraints);
@@ -792,17 +737,17 @@ static int do_search ( int argc, char ** argv, mpd_Connection * conn, int exact
 	return 0;
 }
 
-int cmd_search ( int argc, char ** argv, mpd_Connection * conn ) 
+int cmd_search ( int argc, char ** argv, mpd_Connection * conn )
 {
 	return do_search(argc, argv, conn, 0);
 }
 
-int cmd_find ( int argc, char ** argv, mpd_Connection * conn ) 
+int cmd_find ( int argc, char ** argv, mpd_Connection * conn )
 {
 	return do_search(argc, argv, conn, 1);
 }
 
-int cmd_list ( int argc, char ** argv, mpd_Connection * conn ) 
+int cmd_list ( int argc, char ** argv, mpd_Connection * conn )
 {
 	Constraint *constraints;
 	int numconstraints = 0;
@@ -833,7 +778,7 @@ int cmd_list ( int argc, char ** argv, mpd_Connection * conn )
 	if (argc > 0) {
 		for (i = 0; i < numconstraints; i++) {
 			mpd_addConstraintSearch(conn, constraints[i].type,
-						toUtf8(constraints[i].query));
+						charset_to_utf8(constraints[i].query));
 		}
 
 		free(constraints);
@@ -844,7 +789,7 @@ int cmd_list ( int argc, char ** argv, mpd_Connection * conn )
 
 	while ((tag = mpd_getNextTag(conn, type))) {
 		printErrorAndExit(conn);
-		printf("%s\n", fromUtf8(tag));
+		printf("%s\n", charset_from_utf8(tag));
 		free(tag);
 	}
 
@@ -853,7 +798,7 @@ int cmd_list ( int argc, char ** argv, mpd_Connection * conn )
 	return 0;
 }
 
-int cmd_volume ( int argc, char ** argv, mpd_Connection * conn ) 
+int cmd_volume ( int argc, char ** argv, mpd_Connection * conn )
 {
         struct int_value_change ch;
 
@@ -874,7 +819,7 @@ int cmd_volume ( int argc, char ** argv, mpd_Connection * conn )
 
 	if (ch.is_relative)
 		mpd_sendVolumeCommand(conn,ch.value);
-	else 
+	else
 		mpd_sendSetvolCommand(conn,ch.value);
 
 	my_finishCommand(conn);
@@ -937,6 +882,50 @@ int cmd_random ( int argc, char ** argv, mpd_Connection * conn )
 	return 1;
 }
 
+int cmd_single ( int argc, char ** argv, mpd_Connection * conn )
+{
+	int mode;
+
+	if(argc==1) {
+		mode = get_boolean(argv[0]);
+		if (mode < 0)
+			return -1;
+	}
+	else {
+		mpd_Status * status;
+		status = getStatus(conn);
+		mode = !status->single;
+		mpd_freeStatus(status);
+	}
+
+	mpd_sendSingleCommand(conn,mode);
+	my_finishCommand(conn);
+
+	return 1;
+}
+
+int cmd_consume ( int argc, char ** argv, mpd_Connection * conn )
+{
+	int mode;
+
+	if(argc==1) {
+		mode = get_boolean(argv[0]);
+		if (mode < 0)
+			return -1;
+	}
+	else {
+		mpd_Status * status;
+		status = getStatus(conn);
+		mode = !status->consume;
+		mpd_freeStatus(status);
+	}
+
+	mpd_sendConsumeCommand(conn,mode);
+	my_finishCommand(conn);
+
+	return 1;
+}
+
 int cmd_crossfade ( int argc, char ** argv, mpd_Connection * conn )
 {
 	int seconds;
@@ -986,7 +975,7 @@ int cmd_loadtab ( int argc, char ** argv, mpd_Connection * conn )
 				if(strncmp(pl->path,argv[0],
 							strlen(argv[0]))==0) {
 					printf("%s\n",
-							fromUtf8(pl->path));
+					       charset_from_utf8(pl->path));
 				}
 			}
 		}
@@ -1000,7 +989,6 @@ int cmd_loadtab ( int argc, char ** argv, mpd_Connection * conn )
 int cmd_lstab ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
-	char * sp;
 	mpd_Directory * dir;
 
 	mpd_sendListallCommand(conn,"");
@@ -1009,14 +997,11 @@ int cmd_lstab ( int argc, char ** argv, mpd_Connection * conn )
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_DIRECTORY) {
 			dir = entity->info.directory;
-			sp = dir->path;
-			/* replace all ' ' with '_' */
-			while((sp = strchr(sp,' '))) *sp = '_';
 			if(argc==1) {
 				if(strncmp(dir->path,argv[0],
 							strlen(argv[0]))==0) {
 					printf("%s\n",
-							fromUtf8(dir->path));
+					       charset_from_utf8(dir->path));
 				}
 			}
 		}
@@ -1031,7 +1016,6 @@ int cmd_lstab ( int argc, char ** argv, mpd_Connection * conn )
 int cmd_tab ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
-	char * sp;
 	struct mpd_song *song;
 
 	mpd_sendListallCommand(conn,"");
@@ -1040,17 +1024,14 @@ int cmd_tab ( int argc, char ** argv, mpd_Connection * conn )
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
 			song = entity->info.song;
-			sp = song->file;
-			/* replace all ' ' with '_' */
-			while((sp = strchr(sp,' '))) *sp = '_';
 			if(argc==1) {
 				if(strncmp(song->file,argv[0],
 							strlen(argv[0]))==0) {
 					printf("%s\n",
-							fromUtf8(song->file));
+					       charset_from_utf8(song->file));
 				}
-			}
-			else printf("%s\n",fromUtf8(song->file));
+			} else
+				printf("%s\n", charset_from_utf8(song->file));
 		}
 		mpd_freeInfoEntity(entity);
 	}
@@ -1120,3 +1101,21 @@ cmd_status(mpd_unused  int argc, mpd_unused char **argv, mpd_Connection *conn)
 	print_status(conn);
 	return 0;
 }
+
+int cmd_idle(mpd_unused int argc, mpd_unused char **argv,
+	     mpd_Connection *connection)
+{
+	const char *change;
+
+	mpd_send_idle(connection);
+	printErrorAndExit(connection);
+
+	while ((change = mpd_get_next_idle_change(connection)) != NULL)
+		printf("%s\n", change);
+	printErrorAndExit(connection);
+
+	mpd_finishCommand(connection);
+	printErrorAndExit(connection);
+
+	return 0;
+}
