@@ -1,20 +1,24 @@
-/* mpc
- * (c) 2003-2004 by normalperson and Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #ifndef COMMAND_H
 #define COMMAND_H
@@ -22,6 +26,7 @@
 int cmd_status ( int argc, char ** argv, mpd_Connection *conn );
 int cmd_add ( int argc, char ** argv, mpd_Connection * conn ) ;
 int cmd_crop ( int argc, char ** argv, mpd_Connection * conn );
+int cmd_current ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_del ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_play ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_next ( int argc, char ** argv, mpd_Connection * conn );
@@ -45,6 +50,8 @@ int cmd_rm ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_volume ( int argc, char ** argv, mpd_Connection * conn ) ;
 int cmd_repeat ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_random ( int argc, char ** argv, mpd_Connection * conn );
+int cmd_single ( int argc, char ** argv, mpd_Connection * conn );
+int cmd_consume ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_crossfade ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_enable( int argc, char ** argv, mpd_Connection * conn );
 int cmd_disable( int argc, char ** argv, mpd_Connection * conn );
@@ -56,4 +63,7 @@ int cmd_lstab ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_tab ( int argc, char ** argv, mpd_Connection * conn );
 int cmd_stats ( int argc, char ** argv, mpd_Connection *conn );
 int cmd_toggle ( int argc, char ** argv, mpd_Connection *conn );
+
+int cmd_idle(int argc, char **argv, mpd_Connection *connection);
+
 #endif /* COMMAND_H */
