@@ -311,13 +311,15 @@ static int mpd_recv(mpd_Connection *connection)
 	}
 
 	while (1) {
-		ret = mpd_wait(connection);
-		if (ret < 0) {
-			strcpy(connection->errorStr, "connection timeout");
-			connection->error = MPD_ERROR_TIMEOUT;
-			connection->doneProcessing = 1;
-			connection->doneListOk = 0;
-			return -1;
+		if (!connection->idle) {
+			ret = mpd_wait(connection);
+			if (ret < 0) {
+				strcpy(connection->errorStr, "connection timeout");
+				connection->error = MPD_ERROR_TIMEOUT;
+				connection->doneProcessing = 1;
+				connection->doneListOk = 0;
+				return -1;
+			}
 		}
 
 		nbytes = recv(connection->sock,
@@ -638,6 +640,8 @@ void mpd_finishCommand(mpd_Connection * connection) {
 		if(connection->doneListOk) connection->doneListOk = 0;
 		mpd_getNextReturnElement(connection);
 	}
+
+	connection->idle = 0;
 }
 
 static void mpd_finishListOkCommand(mpd_Connection * connection) {
@@ -678,6 +682,8 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 	status->volume = -1;
 	status->repeat = 0;
 	status->random = 0;
+	status->single = 0;
+	status->consume = 0;
 	status->playlist = -1;
 	status->playlistLength = -1;
 	status->state = -1;
@@ -708,6 +714,12 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 		else if(strcmp(re->name,"random")==0) {
 			status->random = atoi(re->value);
 		}
+		else if(strcmp(re->name,"single")==0) {
+			status->single = atoi(re->value);
+		}
+		else if(strcmp(re->name,"consume")==0) {
+			status->consume = atoi(re->value);
+		}
 		else if(strcmp(re->name,"playlist")==0) {
 			status->playlist = strtol(re->value,NULL,10);
 		}
@@ -1117,7 +1129,7 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 			}
 			else if(!entity->info.song->performer &&
 					strcmp(re->name, "Performer") == 0) {
-				entity->info.song->performer = strdup(re->value);
+				entity->info.song->performer = str_pool_dup(re->value);
 			}
 			else if(!entity->info.song->disc &&
 					strcmp(re->name, "Disc") == 0) {
@@ -1315,7 +1327,7 @@ int mpd_sendAddIdCommand(mpd_Connection *connection, const char *file)
 		retval = atoi(string);
 		free(string);
 	}
-	
+
 	return retval;
 }
 
@@ -1510,6 +1522,22 @@ void mpd_sendRandomCommand(mpd_Connection * connection, int randomMode) {
 	free(string);
 }
 
+void mpd_sendSingleCommand(mpd_Connection * connection, int singleMode) {
+	int len = strlen("single")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "single \"%i\"\n", singleMode);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+
+void mpd_sendConsumeCommand(mpd_Connection * connection, int consumeMode) {
+	int len = strlen("consume")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "consume \"%i\"\n", consumeMode);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+
 void mpd_sendSetvolCommand(mpd_Connection * connection, int volumeChange) {
 	int len = strlen("setvol")+2+INTLEN+3;
 	char *string = malloc(len);
@@ -1895,26 +1923,52 @@ void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
 	free(string);
 }
 
-static void mpd_readChanges(mpd_Connection *connection)
+void mpd_send_idle(mpd_Connection *connection)
+{
+	mpd_executeCommand(connection, "idle\n");
+	if (connection->error == MPD_ERROR_SUCCESS)
+		connection->idle = 1;
+}
+
+const char *
+mpd_get_next_idle_change(mpd_Connection *connection)
 {
-	unsigned i;
-	unsigned flags = 0;
 	mpd_ReturnElement *re;
 
-	if (!connection->returnElement) mpd_getNextReturnElement(connection);
+	if (connection->returnElement == NULL)
+		mpd_getNextReturnElement(connection);
 
-	while (connection->returnElement) {
+	while (connection->returnElement != NULL) {
 		re = connection->returnElement;
-		if (re->name &&!strncmp (re->name, "changed", strlen ("changed"))) {
-			for (i = 0; idle_names[i]; ++i) {
-				if (!strcmp (re->value, idle_names[i])) {
-					flags |= (1 << i);
-				}
-			}
+		if (re->name != NULL && strcmp(re->name, "changed") == 0) {
+			connection->returnElement = NULL;
+			return re->value;
 		}
+
 		mpd_getNextReturnElement(connection);
 	}
 
+	return NULL;
+}
+
+static unsigned
+parse_idle_change(const char *change)
+{
+	for (unsigned i = 0; idle_names[i] != NULL; ++i)
+		if (strcmp(change, idle_names[i]) == 0)
+			return 1 << i;
+
+	return 0;
+}
+
+static void mpd_readChanges(mpd_Connection *connection)
+{
+	const char *change;
+	unsigned flags = 0;
+
+	while ((change = mpd_get_next_idle_change(connection)) != NULL)
+		flags |= parse_idle_change(change);
+
 	/* Notifiy application */
 	if (connection->notify_cb && flags)
 		connection->notify_cb (connection, flags, connection->userdata);
@@ -1928,8 +1982,7 @@ void mpd_startIdle(mpd_Connection *connection, mpd_NotificationCb notify_cb, voi
 	if (connection->startIdle)
 		connection->startIdle(connection);
 
-	mpd_executeCommand(connection, "idle\n");
-	connection->idle = 1;
+	mpd_send_idle(connection);
 	connection->notify_cb = notify_cb;
 	connection->userdata = userdata;
 }
