@@ -235,6 +235,10 @@ typedef struct mpd_Status {
 	int repeat;
 	/* 1 if random is on, 0 otherwise */
 	int random;
+	/* 1 if single is on, 0 otherwise */
+	int single;
+	/* 1 if consume is on, 0 otherwise */
+	int consume;
 	/* playlist length */
 	int playlistLength;
 	/* playlist, use this to determine when the playlist has changed */
@@ -507,6 +511,10 @@ void mpd_sendRepeatCommand(mpd_Connection * connection, int repeatMode);
 
 void mpd_sendRandomCommand(mpd_Connection * connection, int randomMode);
 
+void mpd_sendSingleCommand(mpd_Connection * connection, int singleMode);
+
+void mpd_sendConsumeCommand(mpd_Connection * connection, int consumeMode);
+
 void mpd_sendSetvolCommand(mpd_Connection * connection, int volumeChange);
 
 /* WARNING: don't use volume command, its depreacted */
@@ -662,6 +670,11 @@ void mpd_sendPlaylistMoveCommand(mpd_Connection *connection,
 void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
                                    char *playlist, int pos);
 
+void mpd_send_idle(mpd_Connection *connection);
+
+const char *
+mpd_get_next_idle_change(mpd_Connection *connection);
+
 void mpd_startIdle(mpd_Connection *connection, mpd_NotificationCb notify_cb, void *userdata);
 
 void mpd_stopIdle(mpd_Connection *connection);
