@@ -1,25 +1,28 @@
-/* mpc
- * (c) 2003-2004 by Warren Dukes (warren.dukes@gmail.com)
- *                  Daniel Brown (danb@cs.utexas.edu)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include "libmpdclient.h"
 #include "list.h"
-#include "charConv.h"
+#include "charset.h"
 #include "password.h"
 #include "util.h"
 #include "status.h"
@@ -34,7 +37,7 @@
 #include <stdlib.h>
 #include <sys/param.h>
 
-struct _mpc_table {
+static struct command {
 	const char *command;
 	const int min, max;   /* min/max arguments allowed, -1 = unlimited */
 	int pipe;             /**
@@ -53,6 +56,8 @@ struct _mpc_table {
 	/* command,     min, max, pipe, handler,         usage, help */
 	{"add",         0,   -1,  1,    cmd_add,         "<file>", "Add a song to the current playlist"},
 	{"crop",        0,   0,   0,    cmd_crop,        "", "Remove all but the currently playing song"},
+	{ "current", 0, 0, 0, cmd_current,
+	  "", "Show the currently playing song"},
 	{"del",         0,   -1,  1,    cmd_del,         "<position>", "Remove a song from the current playlist"},
 	{"play",        0,   -1,  2,    cmd_play,        "[<position>]", "Start playing at <position> (default: 1)"},
 	{"next",        0,   0,   0,    cmd_next,        "", "Play the next song in the current playlist"},
@@ -77,6 +82,8 @@ struct _mpc_table {
 	{"volume",      0,   1,   0,    cmd_volume,      "[+-]<num>", "Sets volume to <num> or adjusts by [+-]<num>"},
 	{"repeat",      0,   1,   0,    cmd_repeat,      "<on|off>", "Toggle repeat mode, or specify state"},
 	{"random",      0,   1,   0,    cmd_random,      "<on|off>", "Toggle random mode, or specify state"},
+	{"single",      0,   1,   0,    cmd_single,      "<on|off>", "Toggle single mode, or specify state"},
+	{"consume",     0,   1,   0,    cmd_consume,     "<on|off>", "Toggle consume mode, or specify state"},
 	{"search",      2,   -1,  0,    cmd_search,      "<type> <query>", "Search for a song"},
 	{"find",        2,   -1,  0,    cmd_find,        "<type> <query>", "Find a song (exact match)"},
 	{"list",        1,   -1,  0,    cmd_list,        "<type> [<type> <query>]", "Show all tags of <type>"},
@@ -90,6 +97,7 @@ struct _mpc_table {
 	{"tab",         0,   1,   0,    cmd_tab,         "<path>", NULL},
 	/* status was added for pedantic reasons */
 	{"status",      0,   -1,  0,    cmd_status,      "", NULL},
+	{ "idle", 0, 0, 0, cmd_idle, "", "Idle until an event occurs" },
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
 	{ .command = NULL }
 };
@@ -184,26 +192,29 @@ static mpd_Connection * setup_connection (void)
 	return conn;
 }
 
-static int print_status_and_exit (void)
+static struct command *
+find_command(const char *name)
 {
-	mpd_Connection * conn = setup_connection();
-	print_status(conn);
-	mpd_closeConnection(conn);
-	return EXIT_SUCCESS;
+	for (unsigned i = 0; mpc_table[i].command != NULL; ++i)
+		if (strcmp(name, mpc_table[i].command) == 0)
+			return &mpc_table[i];
+
+	return NULL;
 }
 
 /* check arguments to see if they are valid */
-static char ** check_args(int idx, int * argc, char ** argv)
+static char **
+check_args(struct command *command, int * argc, char ** argv)
 {
 	char ** array;
 	int i;
 
-	if ( ( mpc_table[idx].pipe==1 &&
+	if ((command->pipe == 1 &&
 		(2==*argc || (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL) )))
-	|| (mpc_table[idx].pipe==2 && (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL)))
-	){
+	    || (command->pipe == 2 && (3 == *argc &&
+				       0 == strcmp(argv[2],STDIN_SYMBOL)))){
 		*argc = stdinToArgArray(&array);
-		mpc_table[idx].pipe *= -1;
+		command->pipe *= -1;
 	} else {
 		*argc -= 2;
 		array = malloc( (*argc * (sizeof(char *))));
@@ -211,48 +222,78 @@ static char ** check_args(int idx, int * argc, char ** argv)
 			array[i]=argv[i+2];
 		}
 	}
-	if (	(-1!=mpc_table[idx].min && (*argc)<mpc_table[idx].min)
-	||	(-1!=mpc_table[idx].max && (*argc)>mpc_table[idx].max) ) {
-		fprintf(stderr,"usage: %s %s %s\n", argv[0], mpc_table[idx].command,
-			mpc_table[idx].usage);
+	if ((-1 != command->min && *argc < command->min) ||
+	    (-1 != command->max && *argc > command->max)) {
+		fprintf(stderr,"usage: %s %s %s\n", argv[0], command->command,
+			command->usage);
 			exit (EXIT_FAILURE);
 	}
 	return array;
 }
 
+static int
+run(const struct command *command, int argc, char **array)
+{
+	int ret;
+	mpd_Connection *conn;
+
+	conn = setup_connection();
+
+	ret = command->handler(argc, array, conn);
+	if (ret != 0) {
+		struct mpc_option* nostatus = get_option("no-status");
+		if (!nostatus->set)
+			print_status(conn);
+	}
+
+	mpd_closeConnection(conn);
+	return (ret >= 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+}
+
 int main(int argc, char ** argv)
 {
-	int i, ret;
-	const char *cmd;
-	setLocaleCharset();
+	int ret;
+	const char *command_name;
+	struct command *command;
 
 	if(parse_options(&argc, argv) < 0)
 		return print_help(argv[0],NULL);
 
-	if (argc==1)
-		return print_status_and_exit();
+	/* parse command and arguments */
 
-	cmd = argv[1];
-	for (i = 0; mpc_table[i].command; ++i) {
-		if ( ! strcmp(cmd, mpc_table[i].command) ) {
-			char ** array = check_args (i, &argc , argv);
-			mpd_Connection * conn = setup_connection();
+	if (argc >= 2)
+		command_name = argv[1];
+	else {
+		command_name = "status";
 
-			/* not a typo, assignment intended */
-			if( (ret=mpc_table[i].handler(argc, array, conn))) {
-				struct mpc_option* nostatus = get_option ("no-status");
-				if (! nostatus->set)
-					print_status(conn);
-			}
-
-			if (0>mpc_table[i].pipe)
-				free_pipe_array(argc,array);
-
-			free(array);
-			mpd_closeConnection(conn);
-			return (ret >= 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-		}
+		/* this is a hack, so check_args() won't complain; the
+		   arguments won't we used anyway, so this is quite
+		   safe */
+		argc = 2;
 	}
-	return print_help(argv[0],argv[1]);
-}
 
+	command = find_command(command_name);
+	if (command == NULL)
+		return print_help(argv[0], argv[1]);
+
+	argv = check_args(command, &argc, argv);
+
+	/* initialization */
+
+	charset_init(command->pipe >= 0 || isatty(STDIN_FILENO),
+		     isatty(STDOUT_FILENO));
+
+	/* run */
+
+	ret = run(command, argc, argv);
+
+	/* cleanup */
+
+	charset_deinit();
+
+	if (command->pipe < 0)
+		free_pipe_array(argc, argv);
+	free(argv);
+
+	return ret;
+}
