@@ -1,25 +1,28 @@
-/* mpc
- * (c)2003-2004 by Warren Dukes (warren.dukes@gmail.com)
- *                 Daniel Brown (danb@cs.utexas.edu)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include "status.h"
 #include "libmpdclient.h"
-#include "charConv.h"
+#include "charset.h"
 #include "util.h"
 #include "mpc.h"
 
@@ -44,17 +47,17 @@ void print_status (mpd_Connection *conn)
 	status = mpd_getStatus(conn);
 	printErrorAndExit(conn);
 
-	if(status->state == MPD_STATUS_STATE_PLAY || 
-			status->state == MPD_STATUS_STATE_PAUSE) 
+	if(status->state == MPD_STATUS_STATE_PLAY ||
+			status->state == MPD_STATUS_STATE_PAUSE)
 	{
 		float perc;
 
 		mpd_nextListOkCommand(conn);
 		printErrorAndExit(conn);
-			
+
 		while((entity = mpd_getNextInfoEntity(conn))) {
 			struct mpd_song *song = entity->info.song;
-			
+
 			if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
 				mpd_freeInfoEntity(entity);
 				continue;
@@ -109,9 +112,20 @@ void print_status (mpd_Connection *conn)
 	else printf("off   ");
 
 	printf("random: ");
-	if(status->random) printf("on \n");
+	if(status->random) printf("on    ");
+	else printf("off   ");
+
+	printf("single: ");
+	if(status->single) printf("on    ");
+	else printf("off   ");
+
+	printf("consume: ");
+	if(status->consume) printf("on \n");
 	else printf("off\n");
 
+	if (status->error != NULL)
+		printf("ERROR: %s\n", charset_from_utf8(status->error));
+
 	mpd_freeStatus(status);
 }
 
