@@ -1,20 +1,24 @@
-/* mpc
- * (c) 2003-2004 by normalperson and Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+ 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #ifndef STATUS_H
 #define STATUS_H
