@@ -1,25 +1,28 @@
-/* mpc
- * (c) 2003-2005 by normalperson and Warren Dukes (warren.dukes@gmail.com)
- *              and Daniel Brown (danb@cs.utexas.edu)
- * This project's homepage is: http://www.musicpd.org
- *
+/* music player command (mpc)
+ * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
+				Eric Wong <normalperson@yhbt.net>,
+				Daniel Brown <danb@cs.utexas.edu>
+ * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Project homepage: http://musicpd.org
+ 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include "util.h"
 #include "libmpdclient.h"
-#include "charConv.h"
+#include "charset.h"
 #include "list.h"
 #include "mpc.h"
 #include "options.h"
@@ -34,7 +37,7 @@
 void printErrorAndExit(mpd_Connection * conn)
 {
 	if(conn->error) {
-		fprintf(stderr,"error: %s\n",fromUtf8(conn->errorStr));
+		fprintf(stderr,"error: %s\n",charset_from_utf8(conn->errorStr));
 		mpd_closeConnection(conn);
 		exit(EXIT_FAILURE);
 	}
@@ -263,7 +266,7 @@ songToFormatedString(struct mpd_song *song,
 {
 	char * ret = NULL;
 	const char *p, *end;
-	char * temp;
+	const char *temp;
 	int length;
 	int found = 0;
 	int labelFound = 0;
@@ -298,9 +301,10 @@ songToFormatedString(struct mpd_song *song,
 		
 		if (p[0] == '[')
 		{
-			temp = songToFormatedString(song, p+1, &p);
-			if(temp) {
-				ret = appendToString(ret, temp, strlen(temp));
+			char *t = songToFormatedString(song, p+1, &p);
+			if(t != NULL) {
+				ret = appendToString(ret, t, strlen(t));
+				free(t);
 				found = 1;
 			}
 			continue;
@@ -368,40 +372,40 @@ songToFormatedString(struct mpd_song *song,
 		if (*end != '%') {
 			length--;
 		} else if (strncmp("%file%", p, length) == 0) {
-			temp = fromUtf8(song->file);
+			temp = charset_from_utf8(song->file);
 		} else if (strncmp("%artist%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->artist ? fromUtf8(song->artist) : NULL;
+			temp = song->artist ? charset_from_utf8(song->artist) : NULL;
 		} else if (strncmp("%album%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->album ? fromUtf8(song->album) : NULL;
+			temp = song->album ? charset_from_utf8(song->album) : NULL;
 		} else if (strncmp("%track%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->track ? fromUtf8(song->track) : NULL;
+			temp = song->track ? charset_from_utf8(song->track) : NULL;
 		} else if (strncmp("%title%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->title ? fromUtf8(song->title) : NULL;
+			temp = song->title ? charset_from_utf8(song->title) : NULL;
 		} else if (strncmp("%name%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->name ? fromUtf8(song->name) : NULL;
+			temp = song->name ? charset_from_utf8(song->name) : NULL;
 		} else if (strncmp("%date%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->date ? fromUtf8(song->date) : NULL;
+			temp = song->date ? charset_from_utf8(song->date) : NULL;
 		} else if (strncmp("%genre%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->genre ? fromUtf8(song->genre) : NULL;
+			temp = song->genre ? charset_from_utf8(song->genre) : NULL;
 		} else if (strncmp("%composer%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->composer ? fromUtf8(song->composer) : NULL;
+			temp = song->composer ? charset_from_utf8(song->composer) : NULL;
 		} else if (strncmp("%performer%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->performer ? fromUtf8(song->performer) : NULL;
+			temp = song->performer ? charset_from_utf8(song->performer) : NULL;
 		} else if (strncmp("%disc%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->disc ? fromUtf8(song->disc) : NULL;
+			temp = song->disc ? charset_from_utf8(song->disc) : NULL;
 		} else if (strncmp("%comment%", p, length) == 0) {
 			labelFound = 1;
-			temp = song->comment ? fromUtf8(song->comment) : NULL;
+			temp = song->comment ? charset_from_utf8(song->comment) : NULL;
 		} else if (strncmp("%time%", p, length) == 0) {
 			labelFound = 1;
 			if (song->time != MPD_SONG_NO_TIME) {
@@ -409,7 +413,7 @@ songToFormatedString(struct mpd_song *song,
 				snprintf(s, 9, "%d:%02d", song->time / 60, 
 				                          song->time % 60);
 				/* nasty hack to use static buffer */
-				temp = fromUtf8(s);
+				temp = charset_from_utf8(s);
 			}
 		}
 
@@ -462,7 +466,7 @@ void print_filenames(mpd_Connection *conn)
 	while ((entity = mpd_getNextInfoEntity(conn))) {
 		printErrorAndExit(conn);
 		if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
-			printf("%s\n", fromUtf8(entity->info.song->file));
+			printf("%s\n", charset_from_utf8(entity->info.song->file));
 		mpd_freeInfoEntity(entity);
 	}
 }
